import webbrowser

from PySide6.QtGui import QAction
from PySide6.QtWidgets import QMainWindow, QMessageBox


def set_help_menu(ui_we_want_to_set: QMainWindow):
    ui_we_want_to_set.help_menu .help_github_action = QAction("GitHub")
    ui_we_want_to_set.help_menu .help_github_action.triggered.connect(
        lambda: open_web_browser("https://github.com/Integrated-Testing-Environment/je_editor")
    )
    ui_we_want_to_set.help_menu.addAction(ui_we_want_to_set.help_menu .help_github_action)

    ui_we_want_to_set.help_menu .help_doc_action = QAction("Doc")
    ui_we_want_to_set.help_menu .help_doc_action.triggered.connect(
        lambda: open_web_browser("https://je-editor.readthedocs.io/en/latest/")
    )
    ui_we_want_to_set.help_menu.addAction(ui_we_want_to_set.help_menu .help_doc_action)

    ui_we_want_to_set.help_menu .help_about_action = QAction("About")
    ui_we_want_to_set.help_menu .help_about_action.triggered.connect(
        show_about
    )
    ui_we_want_to_set.help_menu.addAction(ui_we_want_to_set.help_menu.help_about_action)


def open_web_browser(url: str):
    webbrowser.open(url=url)


def show_about():
    message_box = QMessageBox()
    message_box.setText(
        """
JEditor
Create by JE-Chen
        """
    )
    message_box.exec()



import os
import pathlib

from PySide6.QtCore import QDir, QFileInfo
from PySide6.QtWidgets import QMainWindow, QFileSystemModel, QTreeView

from je_editor.pyside_ui.auto_save.auto_save_thread import SaveThread
from je_editor.utils.file.open.open_file import read_file


def set_project_treeview(ui_we_want_to_set: QMainWindow):
    ui_we_want_to_set.grid_layout.setColumnStretch(0, 4)
    ui_we_want_to_set.project_treeview_model = QFileSystemModel()
    ui_we_want_to_set.project_treeview_model.setRootPath(QDir.currentPath())
    ui_we_want_to_set.project_treeview = QTreeView()
    ui_we_want_to_set.project_treeview.setModel(ui_we_want_to_set.project_treeview_model)
    ui_we_want_to_set.project_treeview.setRootIndex(
        ui_we_want_to_set.project_treeview_model.index(os.getcwd())
    )
    ui_we_want_to_set.grid_layout.addWidget(ui_we_want_to_set.project_treeview, 0, 0, 0, 1)
    ui_we_want_to_set.project_treeview.clicked.connect(
        lambda: treeview_click(ui_we_want_to_set)
    )


def treeview_click(ui_we_want_to_set):
    clicked_item: QFileSystemModel = ui_we_want_to_set.project_treeview.selectedIndexes()[0]
    file_info: QFileInfo = ui_we_want_to_set.project_treeview.model().fileInfo(clicked_item)
    path = pathlib.Path(file_info.absoluteFilePath())
    if path.is_file():
        file, file_content = read_file(path)
        ui_we_want_to_set.code_edit.setPlainText(
            file_content
        )
        ui_we_want_to_set.current_file = file
        if ui_we_want_to_set.auto_save_thread is None:
            ui_we_want_to_set.auto_save_thread = SaveThread(
                ui_we_want_to_set.current_file,
                ui_we_want_to_set.code_edit.toPlainText()
            )
            ui_we_want_to_set.auto_save_thread.start()
        if ui_we_want_to_set.auto_save_thread is not None:
            ui_we_want_to_set.auto_save_thread.file = ui_we_want_to_set.current_file



from PySide6.QtGui import QAction, QKeySequence, Qt
from PySide6.QtWidgets import QMainWindow, QMessageBox

from je_editor.pyside_ui.code_process.code_exec import exec_manage
from je_editor.pyside_ui.file_dialog.save_file_dialog import choose_file_get_save_filename
from je_editor.pyside_ui.shell_process.shell_exec import shell_manager


def set_run_menu(ui_we_want_to_set: QMainWindow):
    ui_we_want_to_set.run_menu.run_program_action = QAction("Run Program")
    ui_we_want_to_set.run_menu.run_program_action.triggered.connect(
        lambda: run_program(ui_we_want_to_set)
    )
    ui_we_want_to_set.run_menu.run_program_action.setShortcut(
        QKeySequence(Qt.Key.Key_R, Qt.Key.Key_F1)
    )
    ui_we_want_to_set.run_menu.addAction(ui_we_want_to_set.run_menu.run_program_action)
    ui_we_want_to_set.run_menu.run_on_shell_action = QAction("Run On Shell")
    ui_we_want_to_set.run_menu.run_on_shell_action.triggered.connect(
        lambda: shell_exec(ui_we_want_to_set)
    )
    ui_we_want_to_set.run_menu.run_on_shell_action.setShortcut(
        QKeySequence(Qt.Key.Key_R, Qt.Key.Key_F2)
    )
    ui_we_want_to_set.run_menu.addAction(ui_we_want_to_set.run_menu.run_on_shell_action)
    ui_we_want_to_set.run_menu.clean_result_action = QAction("Clean Result")
    ui_we_want_to_set.run_menu.clean_result_action.triggered.connect(
        lambda: clean_result(ui_we_want_to_set)
    )
    ui_we_want_to_set.run_menu.clean_result_action.setShortcut(
        QKeySequence(Qt.Key.Key_R, Qt.Key.Key_F3)
    )
    ui_we_want_to_set.run_menu.addAction(ui_we_want_to_set.run_menu.clean_result_action)

    ui_we_want_to_set.run_menu.stop_program_action = QAction("Stop Program")
    ui_we_want_to_set.run_menu.stop_program_action.triggered.connect(
        stop_program
    )
    ui_we_want_to_set.run_menu.stop_program_action.setShortcut(
        QKeySequence(Qt.Key.Key_R, Qt.Key.Key_F4)
    )
    ui_we_want_to_set.run_menu.addAction(ui_we_want_to_set.run_menu.stop_program_action)
    # Run help menu
    ui_we_want_to_set.run_menu.run_help_menu = ui_we_want_to_set.run_menu.addMenu("Run Help")
    # Run help action
    ui_we_want_to_set.run_menu.run_help_menu.run_help_action = QAction("Run Help")
    ui_we_want_to_set.run_menu.run_help_menu.run_help_action.triggered.connect(
        show_run_help
    )
    ui_we_want_to_set.run_menu.run_help_menu.addAction(ui_we_want_to_set.run_menu.run_help_menu.run_help_action)
    # Shell help action
    ui_we_want_to_set.run_menu.run_help_menu.shell_help_action = QAction("Shell Help")
    ui_we_want_to_set.run_menu.run_help_menu.shell_help_action.triggered.connect(
        show_shell_help
    )
    ui_we_want_to_set.run_menu.run_help_menu.addAction(ui_we_want_to_set.run_menu.run_help_menu.shell_help_action)


def run_program(ui_we_want_to_set: QMainWindow):
    choose_file_get_save_filename(ui_we_want_to_set)
    exec_manage.main_window = ui_we_want_to_set
    exec_manage.later_init()
    exec_manage.exec_code(
        ui_we_want_to_set.current_file
    )


def shell_exec(ui_we_want_to_set: QMainWindow):
    shell_manager.main_window = ui_we_want_to_set
    shell_manager.later_init()
    shell_manager.exec_shell(
        ui_we_want_to_set.code_edit.toPlainText()
    )


def stop_program():
    exec_manage.exit_program()


def clean_result(ui_we_want_to_set: QMainWindow):
    ui_we_want_to_set.code_result.setPlainText("")


def show_run_help():
    message_box = QMessageBox()
    message_box.setText(
        """
If you are unable to run a Python program, please make sure you are not using the default system Python.
(For Windows, you can use the Windows Store or Venv.)
(For Linux & MacOS, you can use Venv.)
        """
    )
    message_box.exec()


def show_shell_help():
    message_box = QMessageBox()
    message_box.setText(
        """
When executing a shell command, if you encounter a decoding error, 
please make sure that the current encoding is consistent with the default encoding of the system shell.
        """
    )
    message_box.exec()



import time
from pathlib import Path
from threading import Thread

from je_editor.utils.file.save.save_file import write_file


class SaveThread(Thread):

    def __init__(self, file_to_save, text_to_write, auto_save=False):
        """
        :param file_to_save: file we want to auto save
        :param text_to_write: text that will be output to file
        :param auto_save: enable auto save or not
        """
        super().__init__()
        self.file = file_to_save
        self.path = None
        self.text_to_write = text_to_write
        self.auto_save = auto_save
        # set daemon
        self.daemon = True

    def run(self):
        # loop and save current edit file
        if self.file is not None:
            self.auto_save = True
            self.path = Path(self.file)
        while self.auto_save and self.file is not None:
            time.sleep(5)
            if self.path.exists() and self.path.is_file():
                write_file(self.file, self.text_to_write)
            else:
                break



from PySide6.QtGui import QAction, QKeySequence, Qt
from PySide6.QtWidgets import QMainWindow

from je_editor.pyside_ui.file_dialog.open_file_dialog import choose_file_get_open_filename
from je_editor.pyside_ui.file_dialog.save_file_dialog import choose_file_get_save_filename


def set_file_menu(ui_we_want_to_set: QMainWindow):
    ui_we_want_to_set.file_menu.open_file_action = QAction("Open File")
    ui_we_want_to_set.file_menu.open_file_action.setShortcut(
        QKeySequence(Qt.Key.Key_F, Qt.Key.Key_O)
    )
    ui_we_want_to_set.file_menu.open_file_action.triggered.connect(
        lambda: choose_file_get_open_filename(parent_qt_instance=ui_we_want_to_set)
    )
    ui_we_want_to_set.file_menu.addAction(ui_we_want_to_set.file_menu.open_file_action)
    ui_we_want_to_set.file_menu.save_file_action = QAction("Save File")
    ui_we_want_to_set.file_menu.save_file_action.setShortcut(
        QKeySequence(Qt.Key.Key_F, Qt.Key.Key_S)
    )
    ui_we_want_to_set.file_menu.save_file_action.triggered.connect(
        lambda: choose_file_get_save_filename(parent_qt_instance=ui_we_want_to_set)
    )
    ui_we_want_to_set.file_menu.addAction(ui_we_want_to_set.file_menu.save_file_action)





from PySide6.QtGui import QAction
from PySide6.QtWidgets import QMainWindow
from yapf.yapflib.yapf_api import FormatCode

from je_editor.utils.json_format.json_process import reformat_json


def set_check_menu(ui_we_want_to_set: QMainWindow):
    # Yapf code check
    ui_we_want_to_set.check_menu.check_python_action = QAction("yapf")
    ui_we_want_to_set.check_menu.check_python_action.setShortcut("Ctrl+y")
    ui_we_want_to_set.check_menu.check_python_action.triggered.connect(
        lambda: check_python_code(
            ui_we_want_to_set
        )
    )
    ui_we_want_to_set.check_menu.addAction(ui_we_want_to_set.check_menu.check_python_action)
    # Reformat JSON
    ui_we_want_to_set.check_menu.reformat_json_action = QAction("Reformat JSON")
    ui_we_want_to_set.check_menu.reformat_json_action.setShortcut("Ctrl+j")
    ui_we_want_to_set.check_menu.reformat_json_action.triggered.connect(
        lambda: reformat_json_text(
            ui_we_want_to_set
        )
    )
    ui_we_want_to_set.check_menu.addAction(ui_we_want_to_set.check_menu.reformat_json_action)


def check_python_code(ui_we_want_to_set):
    code_text = ui_we_want_to_set.code_edit.toPlainText()
    ui_we_want_to_set.code_result.setPlainText("")
    format_code = FormatCode(
        unformatted_source=code_text,
        verify=True,
        style_config="google"
    )
    if isinstance(format_code, tuple):
        ui_we_want_to_set.code_edit.setPlainText(format_code[0])


def reformat_json_text(ui_we_want_to_set):
    code_text = ui_we_want_to_set.code_edit.toPlainText()
    ui_we_want_to_set.code_result.setPlainText("")
    ui_we_want_to_set.code_edit.setPlainText(reformat_json(code_text))



from PySide6.QtWidgets import QMainWindow, QScrollArea, QGridLayout, QWidget

from je_editor.pyside_ui.code_editor.code_edit_plaintext import CodeEditor
from je_editor.pyside_ui.code_result.code_record import CodeRecord


def set_ui(ui_we_want_to_set: QMainWindow):
    # set qt window
    ui_we_want_to_set.main_widget = QWidget()
    ui_we_want_to_set.grid_layout = QGridLayout(ui_we_want_to_set.main_widget)
    ui_we_want_to_set.grid_layout.setContentsMargins(0, 0, 0, 0)
    ui_we_want_to_set.setCentralWidget(ui_we_want_to_set.main_widget)
    ui_we_want_to_set.setWindowTitle("JEditor")
    # code edit and code result plaintext
    ui_we_want_to_set.code_edit = CodeEditor()
    ui_we_want_to_set.code_result = CodeRecord()
    ui_we_want_to_set.code_edit_scroll_area = QScrollArea()
    ui_we_want_to_set.code_edit_scroll_area.setWidgetResizable(True)
    ui_we_want_to_set.code_edit_scroll_area.setViewportMargins(0, 0, 0, 0)
    ui_we_want_to_set.code_edit_scroll_area.setWidget(ui_we_want_to_set.code_edit)
    ui_we_want_to_set.code_result_scroll_area = QScrollArea()
    ui_we_want_to_set.code_result_scroll_area.setWidgetResizable(True)
    ui_we_want_to_set.code_result_scroll_area.setViewportMargins(0, 0, 0, 0)
    ui_we_want_to_set.code_result_scroll_area.setWidget(ui_we_want_to_set.code_result)
    ui_we_want_to_set.grid_layout.setRowStretch(0, 10)
    ui_we_want_to_set.grid_layout.setColumnStretch(1, 10)
    ui_we_want_to_set.grid_layout.addWidget(ui_we_want_to_set.code_edit_scroll_area, 0, 1)
    ui_we_want_to_set.grid_layout.addWidget(ui_we_want_to_set.code_result_scroll_area, 1, 1)




from PySide6.QtWidgets import QMainWindow, QMenuBar

from je_editor.pyside_ui.menu.menu_bar.check_style_menu.build_check_style_menu import set_check_menu
from je_editor.pyside_ui.menu.menu_bar.file_menu.build_file_menu import set_file_menu
from je_editor.pyside_ui.menu.menu_bar.help_menu.build_help_menu import set_help_menu
from je_editor.pyside_ui.menu.menu_bar.run_menu.build_run_menu import set_run_menu


def set_menu_bar(ui_we_want_to_set: QMainWindow):
    # set menu
    ui_we_want_to_set.menu = QMenuBar()
    ui_we_want_to_set.file_menu = ui_we_want_to_set.menu.addMenu("File")
    ui_we_want_to_set.run_menu = ui_we_want_to_set.menu.addMenu("Run")
    ui_we_want_to_set.text_menu = ui_we_want_to_set.menu.addMenu("Text")
    ui_we_want_to_set.check_menu = ui_we_want_to_set.menu.addMenu("Check Code Style")
    ui_we_want_to_set.help_menu = ui_we_want_to_set.menu.addMenu("Help")
    ui_we_want_to_set.setMenuBar(ui_we_want_to_set.menu)
    set_file_menu(ui_we_want_to_set)
    set_run_menu(ui_we_want_to_set)
    set_check_menu(ui_we_want_to_set)
    set_help_menu(ui_we_want_to_set)



from PySide6.QtWidgets import QWidget, QLineEdit, QPushButton, QBoxLayout, QHBoxLayout


class SearchBox(QWidget):

    def __init__(self):
        super().__init__()
        self.box_layout = QBoxLayout(QBoxLayout.Direction.TopToBottom)
        self.search_input = QLineEdit()
        self.search_next_button = QPushButton()
        self.search_next_button.setText("Search Next")
        self.search_back_button = QPushButton()
        self.search_back_button.setText("Search Back")
        self.box_h_layout = QHBoxLayout()
        self.box_h_layout.addWidget(self.search_back_button)
        self.box_h_layout.addWidget(self.search_next_button)
        self.box_layout.addWidget(self.search_input)
        self.box_layout.addLayout(self.box_h_layout)
        self.setWindowTitle("Search Text")
        self.setLayout(self.box_layout)



import logging
import queue
import sys


class RedirectStdOut(logging.Handler):

    # redirect logging std output to queue

    def __init__(self):
        super().__init__()

    def write(self, content_to_write):
        redirect_manager_instance.std_out_queue.put(content_to_write)

    def emit(self, record: logging.LogRecord) -> None:
        redirect_manager_instance.std_out_queue.put(self.format(record))


class RedirectStdErr(logging.Handler):

    # redirect logging stderr output to queue

    def __init__(self):
        super().__init__()

    def write(self, content_to_write):
        redirect_manager_instance.std_err_queue.put(content_to_write)

    def emit(self, record: logging.LogRecord) -> None:
        redirect_manager_instance.std_err_queue.put(self.format(record))


class RedirectManager(object):

    def __init__(self):
        self.is_use_ui: bool = False
        self.ui = None
        self.std_err_queue = queue.Queue()
        self.std_out_queue = queue.Queue()

    def set_redirect(self, ui, is_use_ui: bool = False):
        """
        :param ui: tkinter_ui instance
        :param is_use_ui: false for cli
        :return: None
        """
        self.ui = ui
        self.is_use_ui = is_use_ui
        if self.is_use_ui is True and self.ui is not None:
            redirect_out = RedirectStdOut()
            redirect_err = RedirectStdErr()
            sys.stdout = redirect_out
            sys.stderr = redirect_err
            default_logger = logging.getLogger()
            default_logger.addHandler(redirect_err)
            for name in logging.root.manager.loggerDict.keys():
                logging.getLogger(name).addHandler(redirect_err)

    @staticmethod
    def restore_std():
        """
        reset redirect
        :return: None
        """
        sys.stdout = sys.__stdout__
        sys.stderr = sys.__stderr__


redirect_manager_instance = RedirectManager()



from PySide6.QtCore import QRegularExpression
from PySide6.QtGui import QColor
from PySide6.QtGui import QFont
from PySide6.QtGui import QSyntaxHighlighter
from PySide6.QtGui import QTextCharFormat

keywords = [
    "False", "None", "True", "and", "as", "assert", "async", 
    "await", "break", "class", "continue", "def", "del",
    "elif", "else", "except", "finally", "for", "from", 
    "global", "if", "import", "in", "is", "lambda", "nonlocal",
    "not", "or", "pass", "raise", "return", "try", "while", "with", "yield"
]

builtins_keyword = [
    "abs", "aiter", "all", "any", "anext", "ascii",
    "bin", "bool", "breakpoint", "bytearray", "bytes",
    "callable", "chr", "classmethod", "compile", "complex",
    "delattr", "dict", "dir", "divmod",
    "enumerate", "eval", "exec",
    "filter", "float", "format", "frozenset",
    "getattr", "globals",
    "hasattr", "hash", "help", "hex",
    "id", "input", "int", "isinstance", "issubclass", "iter",
    "len", "list", "locals",
    "map", "max", "memoryview", "min",
    "next",
    "object", "oct", "open", "ord",
    "pow", "print", "property",
    "range", "repr", "reversed", "round",
    "set", "setattr", "slice", "sorted", "staticmethod",
    "str", "sum", "super",
    "tuple", "type",
    "vars",
    "zip",
    "__import__"
]

string_rule = [
    r"'''[^'\\]*(\\.[^'\\]*)*'''",  # 3* Singel
    r'"""[^"\\]*(\\.[^"\\]*)*"""',  # 3* Double
    r"'[^'\\]*(\\.[^'\\]*)*'",  # Singel
    r'"[^"\\]*(\\.[^"\\]*)*"'  # Double
]

number_rule = [
    r"\b[+-]?[0-9]+[lL]?\b",
    r"\b[+-]?0[xX][0-9A-Fa-f]+[lL]?\b",
    r"\b[+-]?[0-9]+(?:\.[0-9]+)?(?:[eE][+-]?[0-9]+)?\b"
]


class PythonHighlighter(QSyntaxHighlighter):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.highlight_rules = []

        text_char_format = QTextCharFormat()
        text_char_format.setForeground(QColor(255, 212, 102))
        text_char_format.setFontWeight(QFont.Bold)
        for word in keywords:
            pattern = QRegularExpression(rf"\b{word}\b")
            self.highlight_rules.append((pattern, text_char_format))

        text_char_format = QTextCharFormat()
        text_char_format.setForeground(QColor(0, 128, 255))
        for rule in number_rule:
            pattern = QRegularExpression(rule)
            self.highlight_rules.append((pattern, text_char_format))

        text_char_format = QTextCharFormat()
        text_char_format.setForeground(QColor(0, 153, 0))
        for rule in string_rule:
            pattern = QRegularExpression(rule)
            self.highlight_rules.append((pattern, text_char_format))

        text_char_format = QTextCharFormat()
        text_char_format.setForeground(QColor(0, 255, 255))
        for word in builtins_keyword:
            pattern = QRegularExpression(rf"\b{word}\b")
            self.highlight_rules.append((pattern, text_char_format))

        text_char_format = QTextCharFormat()
        text_char_format.setForeground(QColor(255, 255, 255))
        pattern = QRegularExpression(r"#[^\n]*")
        self.highlight_rules.append((pattern, text_char_format))

        text_char_format = QTextCharFormat()
        text_char_format.setForeground(QColor(204, 0, 204))
        pattern = QRegularExpression(r"\bself\b")
        self.highlight_rules.append((pattern, text_char_format))

    def highlightBlock(self, text):
        for pattern, format in self.highlight_rules:
            match_iterator = pattern.globalMatch(text)
            while match_iterator.hasNext():
                match = match_iterator.next()
                self.setFormat(match.capturedStart(), match.capturedLength(), format)



import os
import sys
from pathlib import Path

from PySide6.QtCore import QTimer
from PySide6.QtGui import QFontDatabase, QAction, QIcon
from PySide6.QtWidgets import QMainWindow, QSystemTrayIcon

from je_editor.pyside_ui.auto_save.auto_save_thread import SaveThread
from je_editor.pyside_ui.code_process.code_exec import exec_manage
from je_editor.pyside_ui.colors.global_color import error_color, output_color
from je_editor.pyside_ui.main_ui_setting.ui_setting import set_ui
from je_editor.pyside_ui.menu.menu_bar.set_menu_bar import set_menu_bar
from je_editor.pyside_ui.shell_process.shell_exec import shell_manager
from je_editor.pyside_ui.treeview.project_treeview.set_project_treeview import set_project_treeview
from je_editor.pyside_ui.user_setting.user_setting_file import write_user_setting, user_setting_dict, read_user_setting
from je_editor.utils.encodings.python_encodings import python_encodings_list
from je_editor.utils.file.open.open_file import read_file
from je_editor.utils.redirect_manager.redirect_manager_class import redirect_manager_instance


class EditorMain(QMainWindow):

    def __init__(self):
        super(EditorMain, self).__init__()
        # Windows setup
        self.id = "JEditor"
        if sys.platform in ["win32", "cygwin", "msys"]:
            from ctypes import windll
            windll.shell32.SetCurrentProcessExplicitAppUserModelID(self.id)
        # set python buffered
        os.environ["PYTHONUNBUFFERED"] = "1"
        # Auto save thread
        self.auto_save_thread = None
        # current file
        self.current_file = None
        # Font
        self.font_database = QFontDatabase()
        # Color
        # Timer to redirect error or message
        self.redirect_timer = QTimer(self)
        self.redirect_timer.setInterval(1)
        self.redirect_timer.timeout.connect(self.redirect)
        self.redirect_timer.start()
        set_ui(self)
        set_project_treeview(self)
        set_menu_bar(self)
        # Set font and font size menu
        self.add_font_menu()
        self.add_font_size_menu()
        # Set encoding menu
        self.add_encoding_menu()
        if self.current_file is not None and self.auto_save_thread is None:
            self.auto_save_thread = SaveThread(
                self.current_file,
                self.code_edit.code_edit.toPlainText()
            )
            self.auto_save_thread.start()
        # Set Icon
        self.icon_path = Path(os.getcwd() + "/je_driver_icon.ico")
        self.icon = QIcon(str(self.icon_path))
        if self.icon.isNull() is False:
            self.setWindowIcon(self.icon)
            self.system_icon = QSystemTrayIcon()
            self.system_icon.setIcon(self.icon)
        # Put Redirect on last to trace exception
        redirect_manager_instance.set_redirect(self, True)

    def startup_setting(self):
        read_user_setting()
        self.code_edit.setFont(
            self.font_database.font(
                user_setting_dict.get("font", "Lato"),
                "",
                user_setting_dict.get("font_size", 12)
            )
        )
        self.code_result.setFont(
            self.font_database.font(
                user_setting_dict.get("font", "Lato"),
                "",
                user_setting_dict.get("font_size", 12)
            )
        )
        last_file = user_setting_dict.get("last_file", None)
        if last_file is not None:
            last_file_path = Path(last_file)
            if last_file_path.is_file() and last_file_path.exists():
                self.current_file = str(last_file_path)
                self.code_edit.setPlainText(read_file(self.current_file)[1])

    def add_font_menu(self):
        self.font_menu = self.text_menu.addMenu("Font")
        for family in self.font_database.families():
            font_action = QAction(family, parent=self)
            font_action.triggered.connect(self.set_font)
            self.font_menu.addAction(font_action)

    def add_font_size_menu(self):
        self.font_size_menu = self.text_menu.addMenu("Font Size")
        for size in range(12, 38, 2):
            font_action = QAction(str(size), parent=self)
            font_action.triggered.connect(self.set_font_size)
            self.font_size_menu.addAction(font_action)

    def set_font(self):
        self.code_edit.setFont(
            self.font_database.font(
                self.sender().text(),
                "",
                self.code_edit.font().pointSize()
            )
        )
        self.code_result.setFont(
            self.font_database.font(
                self.sender().text(),
                "",
                self.code_result.font().pointSize()
            )
        )
        user_setting_dict.update({"font": self.sender().text()})

    def set_font_size(self):
        self.code_edit.setFont(
            self.font_database.font(
                self.code_edit.font().family(),
                "",
                int(self.sender().text())
            )
        )
        self.code_result.setFont(
            self.font_database.font(
                self.code_result.font().family(),
                "",
                int(self.sender().text())
            )
        )
        user_setting_dict.update({"font_size": int(self.sender().text())})

    def add_encoding_menu(self):
        self.encoding_menu = self.file_menu.addMenu("Encodings")
        for encoding in python_encodings_list:
            encoding_action = QAction(encoding, parent=self)
            encoding_action.triggered.connect(self.set_encoding)
            self.encoding_menu.addAction(encoding_action)

    def set_encoding(self):
        shell_manager.program_encoding = self.sender().text()
        exec_manage.program_encoding = self.sender().text()
        user_setting_dict.update({"encoding": self.sender().text()})

    def redirect(self):
        if self.auto_save_thread is not None:
            self.auto_save_thread.text_to_write = self.code_edit.toPlainText()
        if not redirect_manager_instance.std_out_queue.empty():
            output_message = redirect_manager_instance.std_out_queue.get_nowait()
            output_message = str(output_message).strip()
            if output_message:
                self.code_result.append(output_message)
        self.code_result.setTextColor(error_color)
        if not redirect_manager_instance.std_err_queue.empty():
            error_message = redirect_manager_instance.std_err_queue.get_nowait()
            error_message = str(error_message).strip()
            if error_message:
                self.code_result.append(error_message)
        self.code_result.setTextColor(output_color)

    def closeEvent(self, event) -> None:
        super().closeEvent(event)
        user_setting_dict.update({"last_file": str(self.current_file)})
        write_user_setting()



from PySide6 import QtGui
from PySide6.QtCore import Qt, QRect
from PySide6.QtGui import QPainter, QColor, QTextCharFormat, QTextFormat, QKeyEvent, QAction, QTextDocument
from PySide6.QtWidgets import QPlainTextEdit, QWidget, QTextEdit

from je_editor.pyside_ui.search_ui.search_text_box import SearchBox
from je_editor.pyside_ui.syntax.python_syntax import PythonHighlighter


class CodeEditor(QPlainTextEdit):

    def __init__(self):
        super().__init__()
        self.line_number = LineNumber(self)
        self.blockCountChanged.connect(self.update_line_number_area_width)
        self.updateRequest.connect(self.update_line_number_area)
        self.update_line_number_area_width(0)
        self.textChanged.connect(self.highlight_current_line)
        self.setTabStopDistance(
            QtGui.QFontMetricsF(self.font()).horizontalAdvance(' ') * 4
        )
        self.highlighter = PythonHighlighter(self.document())
        self.highlight_current_line()
        self.setLineWrapMode(self.LineWrapMode.NoWrap)
        # Search Text
        self.search_action = QAction("Search")
        self.search_action.setShortcut("Ctrl+f")
        self.search_action.triggered.connect(
            self.start_search_dialog
        )
        self.addAction(self.search_action)

    def start_search_dialog(self):
        self.search_box = SearchBox()
        self.search_box.search_back_button.clicked.connect(
            self.find_back_text
        )
        self.search_box.search_next_button.clicked.connect(
            self.find_next_text
        )
        self.search_box.show()

    def find_next_text(self):
        if self.search_box.isVisible():
            text = self.search_box.search_input.text()
            self.find(text)

    def find_back_text(self):
        if self.search_box.isVisible():
            text = self.search_box.search_input.text()
            self.find(text, QTextDocument.FindFlag.FindBackward)

    def line_number_paint(self, event):
        painter = QPainter(self.line_number)
        painter.fillRect(event.rect(), QColor(51, 51, 77))
        block = self.firstVisibleBlock()
        block_number = block.blockNumber()
        top = self.blockBoundingGeometry(block).translated(self.contentOffset()).top()
        bottom = top + self.blockBoundingRect(block).height()
        while block.isValid() and top <= event.rect().bottom():
            if block.isVisible() and bottom >= event.rect().top():
                number = str(block_number + 1)
                painter.setPen(QColor(179, 179, 204))
                painter.drawText(
                    0,
                    top,
                    self.line_number.width(),
                    self.fontMetrics().height(),
                    Qt.AlignmentFlag.AlignCenter,
                    number,
                )
            block = block.next()
            top = bottom
            bottom = top + self.blockBoundingRect(block).height()
            block_number += 1

    def line_number_width(self):
        digits = len(str(self.blockCount()))
        space = 10 * digits
        return space

    def update_line_number_area_width(self, value):
        self.setViewportMargins(self.line_number_width(), 0, 0, 0)

    def resizeEvent(self, event):
        QPlainTextEdit.resizeEvent(self, event)
        cr = self.contentsRect()
        self.line_number.setGeometry(
            QRect(cr.left(), cr.top(), self.line_number_width(), cr.height()),
        )

    def update_line_number_area(self, rect, dy):
        if dy:
            self.line_number.scroll(0, dy)
        else:
            self.line_number.update(
                0,
                rect.y(),
                self.line_number.width(),
                rect.height(),
            )
        if rect.contains(self.viewport().rect()):
            self.update_line_number_area_width(0)

    def highlight_current_line(self):
        selections = []
        if not self.isReadOnly():
            formats = QTextCharFormat()
            selection = QTextEdit.ExtraSelection()
            selection.format = formats
            color_of_the_line = QColor(92, 92, 138)
            selection.cursor = self.textCursor()
            selection.cursor.clearSelection()
            selections.append(selection)
            selection.format.setBackground(color_of_the_line)
            selection.format.setProperty(QTextFormat.FullWidthSelection, True)
        self.setExtraSelections(selections)

    def keyPressEvent(self, event):
        key_event = QKeyEvent(event)
        if key_event.modifiers() and Qt.Modifier.SHIFT:
            key = key_event.key()
            if key == Qt.Key.Key_Enter or key == Qt.Key.Key_Return:
                event.ignore()
            else:
                super().keyPressEvent(event)
        else:
            super().keyPressEvent(event)
        self.highlight_current_line()

    def mousePressEvent(self, event) -> None:
        super().mousePressEvent(event)
        self.highlight_current_line()


class LineNumber(QWidget):

    def __init__(self, editor):
        QWidget.__init__(self, parent=editor)
        self.editor = editor

    def paintEvent(self, event):
        self.editor.line_number_paint(event)



import sys

from PySide6.QtWidgets import QApplication
from qt_material import apply_stylesheet

from je_editor.pyside_ui.main_ui.editor_main_ui.main_editor import EditorMain


def start_editor(**kwargs):
    new_editor = QApplication(sys.argv)
    window = EditorMain()
    apply_stylesheet(new_editor, theme='dark_amber.xml')
    window.showMaximized()
    try:
        window.startup_setting()
    except Exception as error:
        print(repr(error))
    sys.exit(new_editor.exec())



from threading import Lock

from je_editor.utils.exception.exceptions import JEditorSaveFileException


def write_file(file, content):
    """
    :param file: file we want to write
    :param content: content write in file
    try
        lock thread
        if file not empty string
            write content to file
    finally
        release lock
    """
    lock = Lock()
    content = str(content)
    try:
        lock.acquire()
        if file != "" and file is not None:
            with open(file, "w+") as file_to_write:
                file_to_write.write(content)
    except JEditorSaveFileException:
        raise JEditorSaveFileException
    finally:
        lock.release()




import os.path
import queue
import shutil
import subprocess
from pathlib import Path
from threading import Thread

from PySide6.QtCore import QTimer
from PySide6.QtWidgets import QMainWindow, QTextEdit

from je_editor.pyside_ui.colors.global_color import error_color, output_color
from je_editor.utils.exception.exception_tags import compiler_not_found_error, je_editor_init_error
from je_editor.utils.exception.exception_tags import file_not_fond_error
from je_editor.utils.exception.exceptions import JEditorExecException, JEditorException


class ExecManager(object):

    def __init__(
            self,
            main_window=None,
            program_language="python",
            program_encoding="utf-8",
            program_buffer=10240000,
    ):
        """
        :param main_window: tkinter main window
        :param program_language: which program language
        :param program_encoding: which encoding
        """
        self.read_program_error_output_from_thread = None
        self.read_program_output_from_thread = None
        self.main_window: QMainWindow = main_window
        self.code_result: [QTextEdit, None] = None
        self.timer: [QTimer, None] = None
        self.still_run_program = True
        self.process = None
        self.run_output_queue = queue.Queue()
        self.run_error_queue = queue.Queue()
        self.program_language = program_language
        self.program_encoding = program_encoding
        self.program_buffer = program_buffer

    def later_init(self):
        if self.main_window is not None:
            self.code_result: QTextEdit = self.main_window.code_result
            self.timer = QTimer(self.main_window)
        else:
            raise JEditorException(je_editor_init_error)

    def exec_code(self, exec_file_name):
        """
        :param exec_file_name: string file will open to run
        :return: if error return result and True else return result and False
        """
        try:
            self.exit_program()
            self.code_result.setPlainText("")
            reformat_os_file_path = os.path.abspath(exec_file_name)
            # detect file is exist
            try:
                if not Path(exec_file_name).exists():
                    raise JEditorExecException(file_not_fond_error)
            except OSError as error:
                raise JEditorExecException(error)
            compiler_path = shutil.which(self.program_language)
            if compiler_path is None and self.program_language == "python":
                venv_path = Path(os.getcwd() + "/venv")
                if venv_path.is_dir() and venv_path.exists():
                    compiler_path = shutil.which(
                        cmd="python3",
                        path=str(venv_path)
                    )
                else:
                    compiler_path = shutil.which(cmd="python3")
            elif compiler_path is None and self.program_language == "python3":
                venv_path = Path(os.getcwd() + "/venv")
                if venv_path.is_dir() and venv_path.exists():
                    compiler_path = shutil.which(
                        cmd="python",
                        path=str(venv_path)
                    )
                else:
                    compiler_path = shutil.which(cmd="python")
            if compiler_path is None:
                raise JEditorExecException(compiler_not_found_error)
            exec_file = reformat_os_file_path
            # run program
            execute_program_list = [compiler_path, exec_file]
            self.process = subprocess.Popen(
                execute_program_list,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                stdin=subprocess.PIPE,
                shell=False
            )
            self.still_run_program = True
            # program output message queue thread
            self.read_program_output_from_thread = Thread(
                target=self.read_program_output_from_process,
                daemon=True
            )
            self.read_program_output_from_thread.start()
            # program error message queue thread
            self.read_program_error_output_from_thread = Thread(
                target=self.read_program_error_output_from_process,
                daemon=True
            )
            self.read_program_error_output_from_thread.start()
            # show which file execute
            self.code_result.append(compiler_path + " " + reformat_os_file_path)
            # start tkinter_ui update
            # start timer
            self.timer = QTimer(self.main_window)
            self.timer.setInterval(1)
            self.timer.timeout.connect(self.pull_text)
            self.timer.start()
        except Exception as error:
            self.code_result.setTextColor(error_color)
            self.code_result.append(str(error))
            self.code_result.setTextColor(output_color)

    # tkinter_ui update method
    def pull_text(self):
        try:
            self.code_result.setTextColor(error_color)
            if not self.run_error_queue.empty():
                error_message = self.run_error_queue.get_nowait()
                error_message = str(error_message).strip()
                if error_message:
                    self.code_result.append(error_message)
            self.code_result.setTextColor(output_color)
            if not self.run_output_queue.empty():
                output_message = self.run_output_queue.get_nowait()
                output_message = str(output_message).strip()
                if output_message:
                    self.code_result.append(output_message)
        except queue.Empty:
            pass
        if self.process.returncode == 0:
            self.exit_program()
        elif self.process.returncode is not None:
            self.exit_program()
            self.timer.stop()
        if self.still_run_program:
            # poll return code
            self.process.poll()

    # exit program change run flag to false and clean read thread and queue and process
    def exit_program(self):
        self.still_run_program = False
        if self.read_program_output_from_thread is not None:
            self.read_program_output_from_thread = None
        if self.read_program_error_output_from_thread is not None:
            self.read_program_error_output_from_thread = None
        self.print_and_clear_queue()
        if self.process is not None:
            self.process.terminate()

    def print_and_clear_queue(self):
        try:
            for std_output in iter(self.run_output_queue.get_nowait, None):
                std_output = str(std_output).strip()
                if std_output:
                    self.code_result.append(std_output)
            self.code_result.setTextColor(error_color)
            for std_err in iter(self.run_error_queue.get_nowait, None):
                std_err = str(std_err).strip()
                if std_err:
                    self.code_result.append(std_err)
            self.code_result.setTextColor(output_color)
        except queue.Empty:
            pass
        self.run_output_queue = queue.Queue()
        self.run_error_queue = queue.Queue()

    def read_program_output_from_process(self):
        while self.still_run_program:
            program_output_data = self.process.stdout.raw.read(self.program_buffer).decode(self.program_encoding)
            self.run_output_queue.put_nowait(program_output_data)

    def read_program_error_output_from_process(self):
        while self.still_run_program:
            program_error_output_data = self.process.stderr.raw.read(self.program_buffer).decode(self.program_encoding)
            self.run_error_queue.put_nowait(program_error_output_data)


exec_manage = ExecManager()



from PySide6.QtGui import QTextDocument, QAction
from PySide6.QtWidgets import QTextEdit

from je_editor.pyside_ui.search_ui.search_error_box import SearchResultBox


class CodeRecord(QTextEdit):

    def __init__(self):
        super().__init__()
        self.setLineWrapMode(self.LineWrapMode.NoWrap)
        self.setReadOnly(True)
        # Search Result
        self.search_result_action = QAction("Search Error")
        self.search_result_action.setShortcut("Ctrl+e")
        self.search_result_action.triggered.connect(
            self.start_search_result_dialog
        )
        self.addAction(self.search_result_action)

    def start_search_result_dialog(self):
        self.search_result_box = SearchResultBox()
        self.search_result_box.search_back_button.clicked.connect(
            self.find_back_text
        )
        self.search_result_box.search_next_button.clicked.connect(
            self.find_next_text
        )
        self.search_result_box.show()

    def find_next_text(self):
        if self.search_result_box.isVisible():
            text = self.search_result_box.search_input.text()
            self.find(text)

    def find_back_text(self):
        if self.search_result_box.isVisible():
            text = self.search_result_box.search_input.text()
            self.find(text, QTextDocument.FindFlag.FindBackward)



"""
default exception
"""
je_editor_error: str = "JEditor error"
"""
init exception
"""
je_editor_init_error: str = "JEditor can't init"
je_editor_init_exec_manager_exception: str = "JEditor can't init program manager"
"""
exec exception
"""
je_editor_exec_error: str = "JEditor exec error"
file_not_fond_error: str = "File not found"
compiler_not_found_error: str = "Compiler not found"
"""
shell exception
"""
je_editor_shell_error: str = "JEditor run on shell error"
"""
file exception
"""
je_editor_save_file_error: str = "JEditor save file error"
je_editor_open_file_error: str = "JEditor open file error"
"""
json exception
"""
cant_reformat_json_error: str = "Can't reformat json is type right?"
wrong_json_data_error: str = "Can't parser json"
cant_find_json_error: str = "cant find json file"
cant_save_json_error: str = "cant save json file"
"""
content data error
"""
content_set_compiler_error: str = "When set compiler using content make an error"
je_editor_content_file_error: str = "JEditor content file error"



import os

from PySide6.QtWidgets import QFileDialog

from je_editor.utils.file.save.save_file import write_file
from je_editor.pyside_ui.auto_save.auto_save_thread import SaveThread


def choose_file_get_save_filename(parent_qt_instance):
    filename = QFileDialog().getSaveFileName(
        parent=parent_qt_instance,
        dir=os.getcwd()
    )[0]
    if filename is not None and filename != "":
        parent_qt_instance.current_file = filename
        write_file(filename, parent_qt_instance.code_edit.toPlainText())
        if parent_qt_instance.auto_save_thread is None:
            parent_qt_instance.auto_save_thread = SaveThread(
                parent_qt_instance.current_file,
                parent_qt_instance.code_edit.toPlainText()
            )
            parent_qt_instance.auto_save_thread.start()
        elif parent_qt_instance.auto_save_thread is not None:
            parent_qt_instance.auto_save_thread.file = parent_qt_instance.current_file



# start editor
from je_editor.start_editor import start_editor
# Editor
from je_editor.pyside_ui.main_ui.editor_main_ui.main_editor import EditorMain
# Exceptions
from je_editor.utils.exception.exceptions import JEditorException
from je_editor.utils.exception.exceptions import JEditorExecException
from je_editor.utils.exception.exceptions import JEditorRunOnShellException
from je_editor.utils.exception.exceptions import JEditorSaveFileException
from je_editor.utils.exception.exceptions import JEditorOpenFileException
from je_editor.utils.exception.exceptions import JEditorContentFileException
from je_editor.utils.exception.exceptions import JEditorCantFindLanguageException
from je_editor.utils.exception.exceptions import JEditorJsonException
# Color
from je_editor.pyside_ui.colors.global_color import error_color
from je_editor.pyside_ui.colors.global_color import output_color
# Exec and shell
from je_editor.pyside_ui.code_process.code_exec import exec_manage
from je_editor.pyside_ui.shell_process.shell_exec import shell_manager

__all__ = [
    "start_editor", "EditorMain",
    "JEditorException", "JEditorExecException",
    "JEditorRunOnShellException", "JEditorSaveFileException",
    "JEditorOpenFileException", "JEditorContentFileException",
    "JEditorCantFindLanguageException", "JEditorJsonException",
    "error_color", "output_color",
    "exec_manage", "shell_manager"
]
